#version 150
precision highp float; precision highp int;
//Ethan Alexander Shulman 2023 "The Non Absolute" October 5, 2023

in vec3 in_Position;
out vec4 uv;
out float time;

layout(std140) uniform ShaderviewBlock {
	vec4 mouse;
	float screenX,screenY;
	int frame,framerate;
};

#define AA_SAMPLES 100000
//#define ANIMATION 1

#define PI 3.141592653589793


mat2 r2d(float a) {
	float c = cos(a), s = sin(a);
	return mat2(c,s,-s,c);
}
//r2 hash http://extremelearning.com.au/unreasonable-effectiveness-of-quasirandom-sequences/
vec2 quasiRandom(float n) {
	#define R2V 1.32471795724474602596
	return fract(.5+n*(1./vec2(R2V,R2V*R2V)));
}


uniform sampler2D Buffer0;

void vert() {
	#ifdef ANIMATION
	//0-60
	time = float(frame)/float(framerate*AA_SAMPLES)*.3-10.;
	#else
	//screenshot 1
	time = -5.;
	
	//screenshot 2
	//time = -1.;
	
	//screenshot 3
	//time = 4.5;
	#endif
	
	uv = vec4((in_Position.xy*vec2(screenX/screenY,1)+(quasiRandom(float(frame%AA_SAMPLES))-.5)*2./screenY),
		(in_Position.xy*.5+.5)*vec2(screenX,screenY));
	gl_Position = vec4(in_Position,1);
}
void frag() {
	vec2 p = uv.xy;
	p = (p*2.)-vec2(0,-.1);
	
	vec3 c = vec3(0);
	float l;
	#define WE(o,s,t,a,d) l = length(vec2(p.x*d,p.y)-o); c += t*pow(cos(l*s+time*l)*.5+.5,2.2)/(1.+l*l*a);
	#define W(o,s,t,a) WE(o,s,t,a,1.);WE(o,s,t,a,-1.);
	W(vec2(.3,.5), 8., vec3(1,1,.4)*.05, 15.);
	W(vec2(.24,.5), 14., vec3(1,1,.4)*.05, 30.);
	
	W(vec2(.01,.8), 30., vec3(1,1,.8)*.05, 60.);
	W(vec2(0,-.15), 40., vec3(.02,.5,1)*.05, 60.);
	
	W(vec2(.2,-.5), 20., vec3(.5,.5,.5)*.1, 100.);
	W(vec2(.45,-.35), 14., vec3(.8,.5,.1)*.1, 25.);
	W(vec2(.35,-.6), 16., vec3(.8,.4,.05)*.04, 8.);

	vec4 sum = texelFetch(Buffer0,ivec2(uv.zw),0);
	if (frame%AA_SAMPLES == 0) sum = vec4(0);
	sum += vec4(c, 1.);
	gl_FragColor = sum;
}
